@@ -15,6 +15,11 @@ if [ "$GITHUB_EVENT_NAME" == "pull_request" ]; then
 	else
 		SUFFIX="$GITHUB_BASE_REF-pr$PR_NUMBER"
 	fi
+else
+	BRANCH=$(echo $GITHUB_REF | awk 'BEGIN { FS = "/" } ; { print $3 }')
+	if [ "$BRANCH" != "master" ]; then
+		SUFFIX="$BRANCH"
+	fi
 fi
 if [ -n "$SUFFIX" ]; then
 	./bootstrap.ci -s "$SUFFIX"
@@ -50,6 +55,9 @@ else
 	# normal procedure
 
 	CONFIGURE_FLAGS="--disable-dependency-tracking --enable-doc"
+	if [ "$1" != "clang-tidy" ]; then
+		CONFIGURE_FLAGS="$CONFIGURE_FLAGS CLANGTIDY=/bin/no-clang-tidy"
+	fi
 	if [ "$1" == "piv-sm" ]; then
 		CONFIGURE_FLAGS="$CONFIGURE_FLAGS --enable-piv-sm"
 	fi
@@ -59,6 +67,9 @@ else
 	if [ "$1" == "no-shared" ]; then
 		CONFIGURE_FLAGS="$CONFIGURE_FLAGS --disable-shared"
 	fi
+	if [ "$1" == "no-openssl" ]; then
+		CONFIGURE_FLAGS="$CONFIGURE_FLAGS --disable-openssl"
+	fi
 	export CFLAGS="-DDEBUG_PROFILE=1 $CFLAGS"
 	./configure $CONFIGURE_FLAGS
 	make -j 4 V=1
