@@ -2,7 +2,7 @@
 
 set -ex -o xtrace
 
-V=libressl-3.4.2
+V=libressl-3.8.1
 
 sudo apt-get remove -y libssl-dev
 
@@ -10,10 +10,14 @@ if [ ! -d "$V" ]; then
 	# letsencrypt CA does not seem to be included in CI runner
 	wget --no-check-certificate https://ftp.openbsd.org/pub/OpenBSD/LibreSSL/$V.tar.gz
 	tar xzf $V.tar.gz
+
+	pushd $V
+	./configure --prefix=/usr/local
+	make -j $(nproc)
+	popd
 fi
+
 pushd $V
-./configure --prefix=/usr/local
-make -j $(nproc)
 sudo make install
 popd
 
