@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,6 +42,10 @@
 #include <sys/time.h>
 #endif
 
+#ifdef ENABLE_ZLIB
+#include "compression.h"
+#endif
+
 static const struct sc_asn1_entry c_asn1_twlabel[] = {
 	{ "twlabel", SC_ASN1_UTF8STRING, SC_ASN1_TAG_UTF8STRING, 0, NULL, NULL },
 	{ NULL, 0, 0, 0, NULL, NULL }
@@ -528,7 +532,7 @@ sc_pkcs15_get_lastupdate(struct sc_pkcs15_card *p15card)
 	struct sc_context *ctx  = p15card->card->ctx;
 	struct sc_file *file = NULL;
 	struct sc_asn1_entry asn1_last_update[C_ASN1_LAST_UPDATE_SIZE];
-	unsigned char *content, last_update[32];
+	unsigned char *content, last_update[32] = {0};
 	size_t lupdate_len = sizeof(last_update) - 1;
 	int r, content_len;
 	size_t size;
@@ -544,18 +548,19 @@ sc_pkcs15_get_lastupdate(struct sc_pkcs15_card *p15card)
 		return NULL;
 
 	size = file->size ? file->size : 1024;
+	sc_file_free(file);
 
-	content = calloc(size, 1);
+	content = calloc(1, size);
 	if (!content)
 		return NULL;
 
 	r = sc_read_binary(p15card->card, 0, content, size, 0);
-	if (r < 0)
+	if (r < 0) {
+		free(content);
 		return NULL;
+	}
 	content_len = r;
 
-	sc_file_free(file);
-
 	sc_copy_asn1_entry(c_asn1_last_update, asn1_last_update);
 	sc_format_asn1_entry(asn1_last_update + 0, last_update, &lupdate_len, 0);
 
@@ -564,9 +569,11 @@ sc_pkcs15_get_lastupdate(struct sc_pkcs15_card *p15card)
 	if (r < 0)
 		return NULL;
 
-	p15card->tokeninfo->last_update.gtime = strdup((char *)last_update);
-	if (!p15card->tokeninfo->last_update.gtime)
-		return NULL;
+	if (asn1_last_update[0].flags & SC_ASN1_PRESENT) {
+		p15card->tokeninfo->last_update.gtime = strdup((char *)last_update);
+		if (!p15card->tokeninfo->last_update.gtime)
+			return NULL;
+	}
 done:
 	sc_log(ctx, "lastUpdate.gtime '%s'", p15card->tokeninfo->last_update.gtime);
 	return p15card->tokeninfo->last_update.gtime;
@@ -1073,7 +1080,7 @@ sc_pkcs15_bind_internal(struct sc_pkcs15_card *p15card, struct sc_aid *aid)
 	if (p15card->opts.use_file_cache) {
 		err = sc_pkcs15_read_cached_file(p15card, &tmppath, &buf, &len);
 		if (err == SC_SUCCESS)
-			err = len;
+			err = (int)len;
 	}
 	if (err < 0) {
 		err = sc_read_binary(card, 0, buf, len, 0);
@@ -1147,7 +1154,7 @@ sc_pkcs15_bind_internal(struct sc_pkcs15_card *p15card, struct sc_aid *aid)
 	if (p15card->opts.use_file_cache) {
 		err = sc_pkcs15_read_cached_file(p15card, &tmppath, &buf, &len);
 		if (err == SC_SUCCESS)
-			err = len;
+			err = (int)len;
 	}
 	if (err < 0) {
 		err = sc_read_binary(card, 0, buf, len, 0);
@@ -1212,6 +1219,46 @@ end:
 }
 
 
+const char *pkcs15_get_default_use_file_cache(struct sc_card *card)
+{
+	/* enable file caching by default for cards with static content to avoid
+	 * synchronization problems.
+	 *
+	 * The following list was initialized with the cards that can't be modified
+	 * with OpenSC i.e. which don't have a profile/driver for pkcs15-init. */
+	const char *card_drivers_with_file_cache[] = {
+		"atrust-acos",
+		"belpic",
+		"cac1",
+		"cac",
+		"coolkey",
+		"dnie",
+		"edo",
+		"esteid2018",
+		"flex",
+		"cyberflex",
+		"gemsafeV1",
+		"idprime",
+		"itacns",
+		"jpki",
+		"MaskTech",
+		"mcrd",
+		"myeid",
+		"npa",
+		"nqapplet",
+		"tcos",
+	};
+
+	if (NULL == card || NULL == card->driver || NULL == card->driver->short_name)
+		return "no";
+	for (size_t i = 0; i < (sizeof card_drivers_with_file_cache / sizeof *card_drivers_with_file_cache); i++) {
+		if (0 == strcmp(card->driver->short_name, card_drivers_with_file_cache[i]))
+			return "public";
+	}
+
+	return "no";
+}
+
 int
 sc_pkcs15_bind(struct sc_card *card, struct sc_aid *aid,
 		struct sc_pkcs15_card **p15card_out)
@@ -1221,7 +1268,7 @@ sc_pkcs15_bind(struct sc_card *card, struct sc_aid *aid,
 	scconf_block *conf_block = NULL;
 	int r, emu_first, enable_emu;
 	const char *use_file_cache;
-	const char *private_certificate;
+	const char *pin_protected_certificate, *private_certificate;
 
 	if (card == NULL || p15card_out == NULL) {
 		return SC_ERROR_INVALID_ARGUMENTS;
@@ -1237,17 +1284,16 @@ sc_pkcs15_bind(struct sc_card *card, struct sc_aid *aid,
 
 	p15card->card = card;
 	p15card->opts.use_file_cache = SC_PKCS15_OPTS_CACHE_NO_FILES;
-	use_file_cache = "no";
+	use_file_cache = pkcs15_get_default_use_file_cache(card);
 	p15card->opts.use_pin_cache = 1;
 	p15card->opts.pin_cache_counter = 10;
 	p15card->opts.pin_cache_ignore_user_consent = 0;
-	if(0 == strcmp(ctx->app_name, "tokend")) {
-		private_certificate = "ignore";
-		p15card->opts.private_certificate = SC_PKCS15_CARD_OPTS_PRIV_CERT_IGNORE;
+	if (0 == strcmp(ctx->app_name, "tokend")) {
+		pin_protected_certificate = "ignore";
 	} else {
-		private_certificate = "protect";
-		p15card->opts.private_certificate = SC_PKCS15_CARD_OPTS_PRIV_CERT_PROTECT;
+		pin_protected_certificate = "protect";
 	}
+	private_certificate = "";
 
 	conf_block = sc_get_conf_block(ctx, "framework", "pkcs15", 1);
 	if (conf_block) {
@@ -1256,6 +1302,8 @@ sc_pkcs15_bind(struct sc_card *card, struct sc_aid *aid,
 		p15card->opts.pin_cache_counter = scconf_get_int(conf_block, "pin_cache_counter", p15card->opts.pin_cache_counter);
 		p15card->opts.pin_cache_ignore_user_consent = scconf_get_bool(conf_block, "pin_cache_ignore_user_consent",
 				p15card->opts.pin_cache_ignore_user_consent);
+		pin_protected_certificate = scconf_get_str(conf_block, "pin_protected_certificate", pin_protected_certificate);
+		/* read also the old value to keep backward compatibility */
 		private_certificate = scconf_get_str(conf_block, "private_certificate", private_certificate);
 	}
 
@@ -1267,16 +1315,24 @@ sc_pkcs15_bind(struct sc_card *card, struct sc_aid *aid,
 		p15card->opts.use_file_cache = SC_PKCS15_OPTS_CACHE_NO_FILES;
 	}
 
-	if (0 == strcmp(private_certificate, "protect")) {
-		p15card->opts.private_certificate = SC_PKCS15_CARD_OPTS_PRIV_CERT_PROTECT;
-	} else if (0 == strcmp(private_certificate, "ignore")) {
-		p15card->opts.private_certificate = SC_PKCS15_CARD_OPTS_PRIV_CERT_IGNORE;
-	} else if (0 == strcmp(private_certificate, "declassify")) {
-		p15card->opts.private_certificate = SC_PKCS15_CARD_OPTS_PRIV_CERT_DECLASSIFY;
+	if (0 == strcmp(pin_protected_certificate, "protect")) {
+		p15card->opts.pin_protected_certificate = SC_PKCS15_CARD_OPTS_PRIV_CERT_PROTECT;
+	} else if (0 == strcmp(pin_protected_certificate, "ignore")) {
+		p15card->opts.pin_protected_certificate = SC_PKCS15_CARD_OPTS_PRIV_CERT_IGNORE;
+	} else if (0 == strcmp(pin_protected_certificate, "declassify")) {
+		p15card->opts.pin_protected_certificate = SC_PKCS15_CARD_OPTS_PRIV_CERT_DECLASSIFY;
 	}
-	sc_log(ctx, "PKCS#15 options: use_file_cache=%d use_pin_cache=%d pin_cache_counter=%d pin_cache_ignore_user_consent=%d private_certificate=%d",
-			p15card->opts.use_file_cache, p15card->opts.use_pin_cache,p15card->opts.pin_cache_counter,
-			p15card->opts.pin_cache_ignore_user_consent, p15card->opts.private_certificate);
+	/* overwrite pin_protected_certificate when private_certificate set */
+	if (0 == strcmp(private_certificate, "protect")) {
+		p15card->opts.pin_protected_certificate = SC_PKCS15_CARD_OPTS_PRIV_CERT_PROTECT;
+	} else if (0 == strcmp(private_certificate, "ignore")) {
+		p15card->opts.pin_protected_certificate = SC_PKCS15_CARD_OPTS_PRIV_CERT_IGNORE;
+	} else if (0 == strcmp(private_certificate, "declassify")) {
+		p15card->opts.pin_protected_certificate = SC_PKCS15_CARD_OPTS_PRIV_CERT_DECLASSIFY;
+	}
+	sc_log(ctx, "PKCS#15 options: use_file_cache=%d use_pin_cache=%d pin_cache_counter=%d pin_cache_ignore_user_consent=%d pin_protected_certificate=%d",
+			p15card->opts.use_file_cache, p15card->opts.use_pin_cache, p15card->opts.pin_cache_counter,
+			p15card->opts.pin_cache_ignore_user_consent, p15card->opts.pin_protected_certificate);
 
 	r = sc_lock(card);
 	if (r) {
@@ -1415,7 +1471,7 @@ __sc_pkcs15_search_objects(struct sc_pkcs15_card *p15card, unsigned int class_ma
 			break;
 	}
 
-	return match_count;
+	return (int)match_count;
 }
 
 
@@ -1938,6 +1994,9 @@ sc_pkcs15_free_object(struct sc_pkcs15_object *obj)
 	case SC_PKCS15_TYPE_CERT:
 		sc_pkcs15_free_cert_info((sc_pkcs15_cert_info_t *)obj->data);
 		break;
+	case SC_PKCS15_TYPE_SKEY:
+		sc_pkcs15_free_skey_info((sc_pkcs15_skey_info_t *)obj->data);
+		break;
 	case SC_PKCS15_TYPE_DATA_OBJECT:
 		sc_pkcs15_free_data_info((sc_pkcs15_data_info_t *)obj->data);
 		break;
@@ -2144,6 +2203,10 @@ sc_pkcs15_parse_df(struct sc_pkcs15_card *p15card, struct sc_pkcs15_df *df)
 			sc_log(ctx, "%s: Error adding object", sc_strerror(r));
 			goto ret;
 		}
+		while (bufsize > 0 && *p == 00) {
+			bufsize--;
+			p++;
+		}
 	};
 
 	if (r > 0)
@@ -2351,6 +2414,55 @@ sc_pkcs15_parse_unusedspace(const unsigned char *buf, size_t buflen, struct sc_p
 	return 0;
 }
 
+static int decompress_file(sc_card_t *card, unsigned char *buf, size_t buflen,
+		unsigned char **out, size_t *outlen, unsigned long flags)
+{
+	LOG_FUNC_CALLED(card->ctx);
+#ifdef ENABLE_ZLIB
+	int rv = SC_SUCCESS;
+	int method = 0;
+
+	if (flags & SC_FILE_FLAG_COMPRESSED_GZIP) {
+		method = COMPRESSION_GZIP;
+	} else if (flags & SC_FILE_FLAG_COMPRESSED_ZLIB) {
+		method = COMPRESSION_ZLIB;
+	} else {
+		method = COMPRESSION_AUTO;
+	}
+
+	rv = sc_decompress_alloc(out, outlen, buf, buflen, method);
+	if (rv != SC_SUCCESS) {
+		sc_log(card->ctx,  "Decompression failed: %d", rv);
+		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INVALID_DATA);
+	}
+	LOG_FUNC_RETURN(card->ctx, SC_SUCCESS);
+#else
+	sc_log(card->ctx, "Compression not supported, no zlib");
+	LOG_FUNC_RETURN(card->ctx, SC_ERROR_NOT_SUPPORTED);
+#endif
+}
+
+int
+sc_decode_do53(sc_context_t *ctx, u8 **data, size_t *data_len,
+		const u8 *buf, size_t buflen)
+{
+	struct sc_asn1_entry c_asn1_do53[] = {
+		{ "do53", SC_ASN1_OCTET_STRING, SC_ASN1_APP|0x13, SC_ASN1_ALLOC|SC_ASN1_UNSIGNED, NULL, NULL },
+		{ NULL, 0, 0, 0, NULL, NULL }
+	};
+	struct sc_asn1_entry asn1_do53[2];
+	int r;
+
+	LOG_FUNC_CALLED(ctx);
+	sc_copy_asn1_entry(c_asn1_do53, asn1_do53);
+	sc_format_asn1_entry(asn1_do53, data, data_len, 0);
+
+	r = sc_asn1_decode(ctx, asn1_do53, buf, buflen, NULL, NULL);
+	LOG_TEST_RET(ctx, r, "ASN.1 parsing of do-53 failed");
+
+	LOG_FUNC_RETURN(ctx, SC_SUCCESS);
+}
+
 
 int
 sc_pkcs15_read_file(struct sc_pkcs15_card *p15card, const struct sc_path *in_path,
@@ -2394,22 +2506,50 @@ sc_pkcs15_read_file(struct sc_pkcs15_card *p15card, const struct sc_path *in_pat
 
 		/* Handle the case where the ASN.1 Path object specified
 		 * index and length values */
-		if (in_path->count < 0) {
-			if (file->size)
-				len = file->size;
-			else
-				len = 1024;
-			offset = 0;
-		}
-		else {
-			offset = in_path->index;
-			len = in_path->count;
-			/* Make sure we're within proper bounds */
-			if (offset >= file->size || offset + len > file->size) {
-				r = SC_ERROR_INVALID_ASN1_OBJECT;
+
+		if (file->ef_structure == SC_FILE_EF_LINEAR_VARIABLE) {
+
+			// in_path->index: record_no
+			// in_path->count: ignored!
+
+			if(file->record_length > 0) {
+				if(file->record_length > MAX_FILE_SIZE) {
+					len = MAX_FILE_SIZE;
+					sc_log(ctx, "  record size truncated, encoded length: %"SC_FORMAT_LEN_SIZE_T"u", file->record_length);
+				} else {
+					len = file->record_length;
+				}
+			} else {
+				len = MAX_FILE_SIZE;
+			}
+
+			if ((in_path->index <= 0) || (in_path->index > (int)(file->record_count))) {
+				sc_log(ctx, "  record number out of bounds: %d", in_path->index);
+				r = SC_ERROR_RECORD_NOT_FOUND;
 				goto fail_unlock;
 			}
+
+		} else {
+
+			if (in_path->count < 0) {
+				if (file->size)
+					len = (file->size > MAX_FILE_SIZE)? MAX_FILE_SIZE:file->size;
+				else
+					len = 1024;
+				offset = 0;
+			}
+			else {
+				offset = in_path->index;
+				len = in_path->count;
+				/* Make sure we're within proper bounds */
+				if (offset >= file->size || offset + len > file->size) {
+					r = SC_ERROR_INVALID_ASN1_OBJECT;
+					goto fail_unlock;
+				}
+			}
 		}
+
+		free(data);
 		data = malloc(len);
 		if (data == NULL) {
 			r = SC_ERROR_OUT_OF_MEMORY;
@@ -2426,7 +2566,7 @@ sc_pkcs15_read_file(struct sc_pkcs15_card *p15card, const struct sc_path *in_pat
 				if (l > 256) {
 					l = 256;
 				}
-				r = sc_read_record(p15card->card, i, head, l, SC_RECORD_BY_REC_NR);
+				r = sc_read_record(p15card->card, i, 0, head, l, SC_RECORD_BY_REC_NR);
 				if (r == SC_ERROR_RECORD_NOT_FOUND)
 					break;
 				if (r < 0) {
@@ -2448,13 +2588,36 @@ sc_pkcs15_read_file(struct sc_pkcs15_card *p15card, const struct sc_path *in_pat
 			}
 			len = head-data;
 		}
+		else if (file->ef_structure == SC_FILE_EF_LINEAR_VARIABLE) {
+			r = sc_read_record(p15card->card, in_path->index, (unsigned)offset, data, len, SC_RECORD_BY_REC_NR);
+			if (r < 0) {
+				goto fail_unlock;
+			}
+			/* sc_read_record may return less than requested */
+			len = r;
+		}
 		else {
-			r = sc_read_binary(p15card->card, offset, data, len, 0);
+			unsigned long flags = 0;
+			r = sc_read_binary(p15card->card, (unsigned)offset, data, len, &flags);
 			if (r < 0) {
 				goto fail_unlock;
 			}
 			/* sc_read_binary may return less than requested */
 			len = r;
+
+			if (flags & SC_FILE_FLAG_COMPRESSED_AUTO
+			    || flags & SC_FILE_FLAG_COMPRESSED_ZLIB
+			    || flags & SC_FILE_FLAG_COMPRESSED_GZIP) {
+				unsigned char *decompressed_buf = NULL;
+				size_t decompressed_len = 0;
+				r = decompress_file(p15card->card, data, len, &decompressed_buf, &decompressed_len, flags);
+				if (r != SC_SUCCESS) {
+					goto fail_unlock;
+				}
+				free(data);
+				data = decompressed_buf;
+				len = decompressed_len;
+			}
 		}
 		sc_unlock(p15card->card);
 
@@ -2464,6 +2627,10 @@ sc_pkcs15_read_file(struct sc_pkcs15_card *p15card, const struct sc_path *in_pat
 		    && ((p15card->opts.use_file_cache & SC_PKCS15_OPTS_CACHE_ALL_FILES) || !private_data)) {
 			sc_pkcs15_cache_file(p15card, in_path, data, len);
 		}
+		if (len == 0) {
+			free(data);
+			data = NULL;
+		}
 	}
 	*buf = data;
 	*buflen = len;
@@ -2825,7 +2992,8 @@ sc_pkcs15_get_object_guid(struct sc_pkcs15_card *p15card, const struct sc_pkcs15
 	struct sc_serial_number serialnr;
 	struct sc_pkcs15_id  id;
 	unsigned char guid_bin[SC_PKCS15_MAX_ID_SIZE + SC_MAX_SERIALNR];
-	int rv, guid_bin_size;
+	int rv;
+	size_t guid_bin_size;
 
 	LOG_FUNC_CALLED(ctx);
 	if(!out || !out_size)
