@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
@@ -32,6 +32,10 @@
 #include <sys/time.h>
 #endif
 #include <time.h>
+#include <unistd.h>
+#endif
+#ifdef HAVE_PTHREAD
+#include <pthread.h>
 #endif
 
 #define CRYPTOKI_EXPORTS
@@ -47,6 +51,9 @@ static CK_FUNCTION_LIST_PTR pkcs11_spy = NULL;
 static CK_FUNCTION_LIST_3_0_PTR pkcs11_spy_3_0 = NULL;
 /* Real Module Function List */
 static CK_FUNCTION_LIST_3_0_PTR po = NULL;
+/* Real module interface list */
+static CK_INTERFACE_PTR orig_interfaces = NULL;
+static unsigned long num_orig_interfaces = 0;
 /* Dynamic Module Handle */
 static void *modhandle = NULL;
 /* Spy module output */
@@ -317,14 +324,17 @@ enter(const char *function)
 
 	fprintf(spy_output, "\n%d: %s\n", count++, function);
 #ifdef _WIN32
-        GetLocalTime(&st);
-        fprintf(spy_output, "%i-%02i-%02i %02i:%02i:%02i.%03i\n", st.wYear, st.wMonth, st.wDay,
-			st.wHour, st.wMinute, st.wSecond, st.wMilliseconds);
+	GetLocalTime(&st);
+	fprintf(spy_output, "P:%lu; T:%lu %i-%02i-%02i %02i:%02i:%02i.%03i\n",
+			(unsigned long)GetCurrentProcessId(), (unsigned long)GetCurrentThreadId(),
+			st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond, st.wMilliseconds);
 #else
 	gettimeofday (&tv, NULL);
 	tm = localtime (&tv.tv_sec);
 	strftime (time_string, sizeof(time_string), "%F %H:%M:%S", tm);
-	fprintf(spy_output, "%s.%03ld\n", time_string, (long)tv.tv_usec / 1000);
+	fprintf(spy_output, "P:%lu; T:0x%lu %s.%03ld\n",
+			(unsigned long)getpid(), (unsigned long)pthread_self(),
+			time_string, (long)tv.tv_usec / 1000);
 #endif
 
 }
@@ -443,6 +453,21 @@ spy_dump_mechanism_in(const char *name, CK_MECHANISM_PTR pMechanism)
 			break;
 		}
 		break;
+	case CKM_AES_CCM:
+		if (pMechanism->pParameter != NULL) {
+			CK_CCM_PARAMS *param = (CK_CCM_PARAMS *)pMechanism->pParameter;
+			snprintf(param_name, sizeof(param_name), "%s->pParameter->ulDataLen", name);
+			spy_dump_ulong_in(param_name, param->ulDataLen);
+			snprintf(param_name, sizeof(param_name), "%s->pParameter->pNonce[ulNonceLen]", name);
+			spy_dump_string_in(param_name, param->pNonce, param->ulNonceLen);
+			snprintf(param_name, sizeof(param_name), "%s->pParameter->pAAD[ulAADLen]", name);
+			spy_dump_string_in(param_name, param->pAAD, param->ulAADLen);
+			fprintf(spy_output, "[in] %s->pParameter->ulMacLen = %lu\n", name, param->ulMACLen);
+		} else {
+			fprintf(spy_output, "[in] %s->pParameter = NULL\n", name);
+			break;
+		}
+		break;
 	case CKM_ECDH1_DERIVE:
 	case CKM_ECDH1_COFACTOR_DERIVE:
 		if (pMechanism->pParameter != NULL) {
@@ -689,7 +714,7 @@ C_GetMechanismInfo(CK_SLOT_ID  slotID, CK_MECHANISM_TYPE type,
 
 	enter("C_GetMechanismInfo");
 	spy_dump_ulong_in("slotID", slotID);
-	FPRINTF_LOOKUP_ENUM("[in] type = %s", MEC_T, type);
+	FPRINTF_LOOKUP_ENUM("[in] type = %s\n", MEC_T, type);
 
 	rv = po->C_GetMechanismInfo(slotID, type, pInfo);
 	if(rv == CKR_OK) {
@@ -1013,8 +1038,11 @@ C_Encrypt(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pData, CK_ULONG  ulDataLen,
 	spy_dump_ulong_in("hSession", hSession);
 	spy_dump_string_in("pData[ulDataLen]", pData, ulDataLen);
 	rv = po->C_Encrypt(hSession, pData, ulDataLen, pEncryptedData, pulEncryptedDataLen);
-	if (rv == CKR_OK)
+	if (rv == CKR_OK) {
 		spy_dump_string_out("pEncryptedData[*pulEncryptedDataLen]", pEncryptedData, *pulEncryptedDataLen);
+	} else if (rv == CKR_BUFFER_TOO_SMALL) {
+		spy_dump_ulong_out("pulEncryptedDataLen", *pulEncryptedDataLen);
+	}
 	return retne(rv);
 }
 
@@ -1041,9 +1069,12 @@ C_EncryptFinal(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pLastEncryptedPart, CK_UL
 	enter("C_EncryptFinal");
 	spy_dump_ulong_in("hSession", hSession);
 	rv = po->C_EncryptFinal(hSession, pLastEncryptedPart, pulLastEncryptedPartLen);
-	if (rv == CKR_OK)
+	if (rv == CKR_OK) {
 		spy_dump_string_out("pLastEncryptedPart[*pulLastEncryptedPartLen]", pLastEncryptedPart,
 				*pulLastEncryptedPartLen);
+	} else if (rv == CKR_BUFFER_TOO_SMALL) {
+		spy_dump_ulong_out("pulLastEncryptedPartLen", *pulLastEncryptedPartLen);
+	}
 
 	return retne(rv);
 }
@@ -1071,8 +1102,11 @@ C_Decrypt(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pEncryptedData, CK_ULONG  ulEn
 	spy_dump_ulong_in("hSession", hSession);
 	spy_dump_string_in("pEncryptedData[ulEncryptedDataLen]", pEncryptedData, ulEncryptedDataLen);
 	rv = po->C_Decrypt(hSession, pEncryptedData, ulEncryptedDataLen, pData, pulDataLen);
-	if (rv == CKR_OK)
+	if (rv == CKR_OK) {
 		spy_dump_string_out("pData[*pulDataLen]", pData, *pulDataLen);
+	} else if (rv == CKR_BUFFER_TOO_SMALL) {
+		spy_dump_ulong_out("pulDataLen", *pulDataLen);
+	}
 
 	return retne(rv);
 }
@@ -1101,8 +1135,11 @@ C_DecryptFinal(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pLastPart, CK_ULONG_PTR p
 	enter("C_DecryptFinal");
 	spy_dump_ulong_in("hSession", hSession);
 	rv = po->C_DecryptFinal(hSession, pLastPart, pulLastPartLen);
-	if (rv == CKR_OK)
+	if (rv == CKR_OK) {
 		spy_dump_string_out("pLastPart[*pulLastPartLen]", pLastPart, *pulLastPartLen);
+	} else if (rv == CKR_BUFFER_TOO_SMALL) {
+		spy_dump_ulong_out("pulLastPartLen", *pulLastPartLen);
+	}
 
 	return retne(rv);
 }
@@ -1196,8 +1233,11 @@ C_Sign(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pData, CK_ULONG  ulDataLen,
 	spy_dump_ulong_in("hSession", hSession);
 	spy_dump_string_in("pData[ulDataLen]", pData, ulDataLen);
 	rv = po->C_Sign(hSession, pData, ulDataLen, pSignature, pulSignatureLen);
-	if (rv == CKR_OK)
+	if (rv == CKR_OK) {
 		spy_dump_string_out("pSignature[*pulSignatureLen]", pSignature, *pulSignatureLen);
+	} else if (rv == CKR_BUFFER_TOO_SMALL) {
+		spy_dump_ulong_out("pulSignatureLen", *pulSignatureLen);
+	}
 
 	return retne(rv);
 }
@@ -1222,8 +1262,11 @@ C_SignFinal(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pSignature, CK_ULONG_PTR pul
 	enter("C_SignFinal");
 	spy_dump_ulong_in("hSession", hSession);
 	rv = po->C_SignFinal(hSession, pSignature, pulSignatureLen);
-	if (rv == CKR_OK)
+	if (rv == CKR_OK) {
 		spy_dump_string_out("pSignature[*pulSignatureLen]", pSignature, *pulSignatureLen);
+	} else if (rv == CKR_BUFFER_TOO_SMALL) {
+		spy_dump_ulong_out("pulSignatureLen", *pulSignatureLen);
+	}
 
 	return retne(rv);
 }
@@ -1251,8 +1294,11 @@ C_SignRecover(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pData, CK_ULONG  ulDataLen
 	spy_dump_ulong_in("hSession", hSession);
 	spy_dump_string_in("pData[ulDataLen]", pData, ulDataLen);
 	rv = po->C_SignRecover(hSession, pData, ulDataLen, pSignature, pulSignatureLen);
-	if (rv == CKR_OK)
+	if (rv == CKR_OK) {
 		spy_dump_string_out("pSignature[*pulSignatureLen]", pSignature, *pulSignatureLen);
+	} else if (rv == CKR_BUFFER_TOO_SMALL) {
+		spy_dump_ulong_out("pulSignatureLen", *pulSignatureLen);
+	}
 	return retne(rv);
 }
 
@@ -1455,8 +1501,11 @@ C_WrapKey(CK_SESSION_HANDLE hSession, CK_MECHANISM_PTR pMechanism,
 	spy_dump_ulong_in("hWrappingKey", hWrappingKey);
 	spy_dump_ulong_in("hKey", hKey);
 	rv = po->C_WrapKey(hSession, pMechanism, hWrappingKey, hKey, pWrappedKey, pulWrappedKeyLen);
-	if (rv == CKR_OK)
+	if (rv == CKR_OK) {
 		spy_dump_string_out("pWrappedKey[*pulWrappedKeyLen]", pWrappedKey, *pulWrappedKeyLen);
+	} else if (rv == CKR_BUFFER_TOO_SMALL) {
+		spy_dump_ulong_out("pulWrappedKeyLen", *pulWrappedKeyLen);
+	}
 
 	return retne(rv);
 }
@@ -1595,6 +1644,21 @@ C_GetInterfaceList(CK_INTERFACE_PTR pInterfacesList, CK_ULONG_PTR pulCount)
 	if (po->version.major < 3) {
 		fprintf(spy_output, "[compat]\n");
 
+		if (pulCount == NULL_PTR)
+			return retne(CKR_ARGUMENTS_BAD);
+
+		if (pInterfacesList == NULL_PTR) {
+			*pulCount = NUM_INTERFACES;
+			spy_dump_ulong_out("*pulCount", *pulCount);
+			return retne(CKR_OK);
+		}
+		spy_dump_ulong_in("*pulCount", *pulCount);
+		if (*pulCount < NUM_INTERFACES) {
+			*pulCount = NUM_INTERFACES;
+			spy_dump_ulong_out("*pulCount", *pulCount);
+			return retne(CKR_BUFFER_TOO_SMALL);
+		}
+
 		memcpy(pInterfacesList, compat_interfaces, NUM_INTERFACES * sizeof(CK_INTERFACE));
 		*pulCount = NUM_INTERFACES;
 
@@ -1605,17 +1669,30 @@ C_GetInterfaceList(CK_INTERFACE_PTR pInterfacesList, CK_ULONG_PTR pulCount)
 	}
 	rv = po->C_GetInterfaceList(pInterfacesList, pulCount);
 	if (rv == CKR_OK) {
-		spy_dump_desc_out("pInterfacesList");
+		spy_dump_desc_out("pInterfacesList (original)");
 		print_interfaces_list(spy_output, pInterfacesList, *pulCount);
-		spy_dump_ulong_out("*pulCount", *pulCount);
 
-		/* Now, replace function lists of known interfaces (PKCS 11, v 2.x and 3.0) */
 		if (pInterfacesList != NULL) {
 			unsigned long i;
+			/* Record the module interface so we can transparently proxy the GetInterface calls */
+			free(orig_interfaces);
+			num_orig_interfaces = 0;
+			orig_interfaces = malloc(*pulCount * sizeof(CK_INTERFACE));
+			if (orig_interfaces == NULL) {
+				return CKR_HOST_MEMORY;
+			}
+			memcpy(orig_interfaces, pInterfacesList, *pulCount * sizeof(CK_INTERFACE));
+			num_orig_interfaces = *pulCount;
+
+			/* Now, replace function lists of known interfaces (PKCS 11, v 2.x and 3.0) */
 			for (i = 0; i < *pulCount; i++) {
 				spy_interface_function_list(&pInterfacesList[i]);
 			}
 		}
+
+		spy_dump_desc_out("pInterfacesList (faked)");
+		print_interfaces_list(spy_output, pInterfacesList, *pulCount);
+		spy_dump_ulong_out("*pulCount", *pulCount);
 	}
 	return retne(rv);
 }
@@ -1649,8 +1726,27 @@ C_GetInterface(CK_UTF8CHAR_PTR pInterfaceName, CK_VERSION_PTR pVersion,
 	fprintf(spy_output, "[in] flags = %s\n",
 		(flags & CKF_INTERFACE_FORK_SAFE ? "CKF_INTERFACE_FORK_SAFE" : ""));
 	if (po->version.major >= 3) {
+		/* We can not assume the version we told the caller matches the version in the underlying
+		 * pkcs11 module so map it back to the known ones */
+		CK_VERSION in_version;
+		if ((pInterfaceName == NULL || strcmp((char *)pInterfaceName, "PKCS 11") == 0) && pVersion) {
+			for (unsigned long i = 0; i < num_orig_interfaces; i++) {
+				CK_VERSION *v = (CK_VERSION *)orig_interfaces[i].pFunctionList;
+				/* We found the same major version. Copy the minor and call it a day */
+				if (v->major == pVersion->major) {
+					in_version.major = v->major;
+					in_version.minor = v->minor;
+					pVersion = &in_version;
+					fprintf(spy_output, "[in] pVersion = %d.%d (faked)\n",
+							pVersion->major, pVersion->minor);
+					break;
+				}
+			}
+			/* If not found, see what we will get */
+		}
+
 		rv = po->C_GetInterface(pInterfaceName, pVersion, ppInterface, flags);
-		if (ppInterface != NULL) {
+		if (rv == CKR_OK && ppInterface != NULL) {
 			spy_interface_function_list(*ppInterface);
 		}
 	} else {
