@@ -22,8 +22,10 @@
 #include "p11test_case_common.h"
 #include "../../libopensc/sc-ossl-compat.h"
 
-char name_buffer[11];
-char flag_buffer[11];
+/* Unsigned long can be up to 16 B long. We print also leading "0x" and we need trailing NULL byte */
+#define FLAG_BUFFER_LEN 19
+char name_buffer[FLAG_BUFFER_LEN];
+char flag_buffer[FLAG_BUFFER_LEN];
 
 void test_certs_init(test_certs_t *objects)
 {
@@ -58,15 +60,29 @@ test_cert_t *
 add_object(test_certs_t *objects, CK_ATTRIBUTE key_id, CK_ATTRIBUTE label)
 {
 	test_cert_t *o = NULL;
-	objects->count = objects->count + 1;
-	if (objects->count > objects->alloc_count) {
+	unsigned int i;
+
+	if (objects->count + 1 > objects->alloc_count) {
 		objects->alloc_count += 8;
 		objects->data = realloc(objects->data, objects->alloc_count * sizeof(test_cert_t));
 		if (objects->data == NULL)
 			return NULL;
 	}
 
-	o = &(objects->data[objects->count - 1]);
+	/* SoftHSM is stupid returning objects in random order. Sort here by key ID
+	 * to provide deterministic JSON output */
+	for (i = 0; i < objects->count; i++) {
+		size_t len = MIN(objects->data[i].key_id_size, key_id.ulValueLen);
+		if (memcmp(key_id.pValue, objects->data[i].key_id, len) <= 0) {
+			break;
+		}
+	}
+	if (i < objects->count) {
+		memmove(&objects->data[i + 1], &objects->data[i], (objects->count - i) * sizeof(test_cert_t));
+	}
+	objects->count = objects->count + 1;
+
+	o = &(objects->data[i]);
 	o->private_handle = CK_INVALID_HANDLE;
 	o->public_handle = CK_INVALID_HANDLE;
 	o->always_auth = 0;
@@ -236,7 +252,7 @@ add_supported_mechs(test_cert_t *o)
  * and store related information
  */
 int callback_certificates(test_certs_t *objects,
-	CK_ATTRIBUTE template[], unsigned int template_size, CK_OBJECT_HANDLE object_handle)
+	CK_ATTRIBUTE template[], unsigned long template_size, CK_OBJECT_HANDLE object_handle)
 {
 	EVP_PKEY *evp = NULL;
 	const u_char *cp = NULL;
@@ -245,6 +261,11 @@ int callback_certificates(test_certs_t *objects,
 	if (*(CK_CERTIFICATE_TYPE *)template[3].pValue != CKC_X_509)
 		return 0;
 
+	/* Ignore objects with empty ID -- we don't know what to do with them */
+	if (template[0].ulValueLen == 0) {
+		return 0;
+	}
+
 	if ((o = add_object(objects, template[0], template[2])) == NULL)
 		return -1;
 
@@ -283,11 +304,16 @@ int callback_certificates(test_certs_t *objects,
  * Pair found private keys on the card with existing certificates
  */
 int callback_private_keys(test_certs_t *objects,
-	CK_ATTRIBUTE template[], unsigned int template_size, CK_OBJECT_HANDLE object_handle)
+	CK_ATTRIBUTE template[], unsigned long template_size, CK_OBJECT_HANDLE object_handle)
 {
 	test_cert_t *o = NULL;
 	char *key_id;
 
+	/* Ignore objects with empty ID -- we don't know what to do with them */
+	if (template[3].ulValueLen == 0) {
+		return 0;
+	}
+
 	/* Search for already stored certificate with same ID */
 	if ((o = search_certificate(objects, &(template[3]))) == NULL) {
 		key_id = convert_byte_string(template[3].pValue,
@@ -309,19 +335,19 @@ int callback_private_keys(test_certs_t *objects,
 
 	/* Store attributes, flags and handles */
 	o->private_handle = object_handle;
-	o->sign = (template[0].ulValueLen != (CK_ULONG) -1)
+	o->sign = (template[0].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[0].pValue) : CK_FALSE;
-	o->decrypt = (template[1].ulValueLen != (CK_ULONG) -1)
+	o->decrypt = (template[1].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[1].pValue) : CK_FALSE;
-	o->key_type = (template[2].ulValueLen != (CK_ULONG) -1)
+	o->key_type = (template[2].ulValueLen == sizeof(CK_KEY_TYPE))
 		? *((CK_KEY_TYPE *) template[2].pValue) : (CK_KEY_TYPE) -1;
-	o->always_auth = (template[4].ulValueLen != (CK_ULONG) -1)
+	o->always_auth = (template[4].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[4].pValue) : CK_FALSE;
-	o->unwrap = (template[5].ulValueLen != (CK_ULONG) -1)
+	o->unwrap = (template[5].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[5].pValue) : CK_FALSE;
-	o->derive_priv = (template[6].ulValueLen != (CK_ULONG) -1)
+	o->derive_priv = (template[6].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[6].pValue) : CK_FALSE;
-	o->extractable = (template[8].ulValueLen != (CK_ULONG) -1)
+	o->extractable = (template[8].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[8].pValue) : CK_FALSE;
 
 	debug_print(" [  OK %s ] Private key loaded successfully S:%d D:%d T:%02lX",
@@ -333,7 +359,7 @@ int callback_private_keys(test_certs_t *objects,
  * Pair found public keys on the card with existing certificates
  */
 int callback_public_keys(test_certs_t *objects,
-	CK_ATTRIBUTE template[], unsigned int template_size, CK_OBJECT_HANDLE object_handle)
+	CK_ATTRIBUTE template[], unsigned long template_size, CK_OBJECT_HANDLE object_handle)
 {
 	test_cert_t *o = NULL;
 	char *key_id;
@@ -359,23 +385,23 @@ int callback_public_keys(test_certs_t *objects,
 	}
 
 	o->public_handle = object_handle;
-	o->verify = (template[0].ulValueLen != (CK_ULONG) -1)
+	o->verify = (template[0].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[0].pValue) : CK_FALSE;
-	o->encrypt = (template[1].ulValueLen != (CK_ULONG) -1)
+	o->encrypt = (template[1].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[1].pValue) : CK_FALSE;
 	/* store key type in case there is no corresponding private key */
-	o->key_type = (template[2].ulValueLen != (CK_ULONG) -1)
+	o->key_type = (template[2].ulValueLen == sizeof(CK_KEY_TYPE))
 		? *((CK_KEY_TYPE *) template[2].pValue) : (CK_KEY_TYPE) -1;
-	o->wrap = (template[8].ulValueLen != (CK_ULONG) -1)
+	o->wrap = (template[8].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[8].pValue) : CK_FALSE;
-	o->derive_pub = (template[9].ulValueLen != (CK_ULONG) -1)
+	o->derive_pub = (template[9].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[9].pValue) : CK_FALSE;
 
 	/* check if we get the same public key as from the certificate */
 	if (o->key_type == CKK_RSA) {
 		BIGNUM *n = NULL, *e = NULL;
-		n = BN_bin2bn(template[4].pValue, template[4].ulValueLen, NULL);
-		e = BN_bin2bn(template[5].pValue, template[5].ulValueLen, NULL);
+		n = BN_bin2bn(template[4].pValue, (int)template[4].ulValueLen, NULL);
+		e = BN_bin2bn(template[5].pValue, (int)template[5].ulValueLen, NULL);
 		if (o->key != NULL) {
 			int rv;
 #if OPENSSL_VERSION_NUMBER < 0x30000000L
@@ -390,7 +416,7 @@ int callback_public_keys(test_certs_t *objects,
 				BN_free(cert_n);
 				BN_free(n);
 				BN_free(e);
-				return -1; 
+				return -1;
 			}
 #endif
 			rv = BN_cmp(cert_n, n) == 0 && BN_cmp(cert_e, e) == 0;
@@ -409,11 +435,27 @@ int callback_public_keys(test_certs_t *objects,
 		} else { /* store the public key for future use */
 			o->type = EVP_PKEY_RSA;
 #if OPENSSL_VERSION_NUMBER < 0x30000000L
-			o->key = EVP_PKEY_new();
 			RSA *rsa = RSA_new();
-			if (RSA_set0_key(rsa, n, e, NULL) != 1 ||
-			    EVP_PKEY_set1_RSA(o->key, rsa) != 1) {
-				fail_msg("Unable to set key params");
+			if (rsa == NULL) {
+				fail_msg("Unable to allocate RSA key");
+				return -1;
+			}
+			o->key = EVP_PKEY_new();
+			if (o->key == NULL) {
+				fail_msg("Unable to allocate EVP_PKEY");
+				RSA_free(rsa);
+				return -1;
+			}
+			if (RSA_set0_key(rsa, n, e, NULL) != 1) {
+				fail_msg("Unable set RSA key params");
+				EVP_PKEY_free(o->key);
+				RSA_free(rsa);
+				return -1;
+			}
+			if (EVP_PKEY_assign_RSA(o->key, rsa) != 1) {
+				EVP_PKEY_free(o->key);
+				RSA_free(rsa);
+				fail_msg("Unable to assign RSA to EVP_PKEY");
 				return -1;
 			}
 #else
@@ -439,6 +481,7 @@ int callback_public_keys(test_certs_t *objects,
 			 	fail_msg("Unable to store key");
 				return -1;
 			}
+			EVP_PKEY_CTX_free(ctx);
 			OSSL_PARAM_free(params);
 			BN_free(n);
 			BN_free(e);
@@ -446,13 +489,17 @@ int callback_public_keys(test_certs_t *objects,
 			o->bits = EVP_PKEY_bits(o->key);
 		}
 	} else if (o->key_type == CKK_EC) {
+		int ec_error = 1;
 		ASN1_OBJECT *oid = NULL;
-		ASN1_OCTET_STRING *s = NULL;
+		ASN1_OCTET_STRING *pub_asn1 = NULL;
 		const unsigned char *pub, *p;
-		BIGNUM *bn = NULL;
-		EC_POINT *ecpoint;
+		EC_POINT *ecpoint = NULL;
 		EC_GROUP *ecgroup = NULL;
 		int nid, pub_len;
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+		EC_GROUP *cert_group = NULL;
+		EC_POINT *cert_point = NULL;
+#endif
 
 		/* Parse the nid out of the EC_PARAMS */
 		p = template[6].pValue;
@@ -460,44 +507,36 @@ int callback_public_keys(test_certs_t *objects,
 		if (oid == NULL) {
 			debug_print(" [WARN %s ] Failed to convert EC_PARAMS"
 				" to OpenSSL format", o->id_str);
-			return -1;
+			goto ec_out;
 		}
 		nid = OBJ_obj2nid(oid);
 		ASN1_OBJECT_free(oid);
 		if (nid == NID_undef) {
 			debug_print(" [WARN %s ] Failed to convert EC_PARAMS"
 				" to NID", o->id_str);
-			return -1;
+			goto ec_out;
 		}
 		ecgroup = EC_GROUP_new_by_curve_name(nid);
 		if (ecgroup == NULL) {
 			debug_print(" [WARN %s ] Failed to create new EC_GROUP"
 				" from NID", o->id_str);
-			return -1;
+			goto ec_out;
 		}
 		EC_GROUP_set_asn1_flag(ecgroup, OPENSSL_EC_NAMED_CURVE);
 
 		p = template[7].pValue;
-		s = d2i_ASN1_OCTET_STRING(NULL, &p, template[7].ulValueLen);
-		pub = ASN1_STRING_get0_data(s);
-		pub_len = ASN1_STRING_length(s);
-		bn = BN_bin2bn(pub, pub_len, NULL);
-		ASN1_STRING_free(s);
-		if (bn == NULL) {
-			debug_print(" [WARN %s ] Can not convert EC_POINT from"
-				" PKCS#11 to BIGNUM", o->id_str);
-			EC_GROUP_free(ecgroup);
-			return -1;
+		pub_asn1 = d2i_ASN1_OCTET_STRING(NULL, &p, template[7].ulValueLen);
+		pub = ASN1_STRING_get0_data(pub_asn1);
+		pub_len = ASN1_STRING_length(pub_asn1);
+
+		if (!(ecpoint = EC_POINT_new(ecgroup))) {
+			debug_print(" [WARN %s ] Cannot allocate EC_POINT", o->id_str);
+			goto ec_out;
 		}
 
-		ecpoint = EC_POINT_hex2point(ecgroup, BN_bn2hex(bn), NULL, NULL);
-		BN_free(bn);
-		
-		if (ecpoint == NULL) {
-			debug_print(" [WARN %s ] Can not convert EC_POINT from"
-				" BIGNUM to OpenSSL format", o->id_str);
-			EC_GROUP_free(ecgroup);
-			return -1;
+		if (EC_POINT_oct2point(ecgroup, ecpoint, pub, pub_len, NULL) != 1) {
+			debug_print(" [WARN %s ] Cannot parse EC_POINT", o->id_str);
+			goto ec_out;
 		}
 
 		if (o->key != NULL) {
@@ -507,30 +546,23 @@ int callback_public_keys(test_certs_t *objects,
 			const EC_POINT *cert_point = EC_KEY_get0_public_key(ec);
 			int cert_nid = EC_GROUP_get_curve_name(cert_group);
 #else
-			EC_GROUP *cert_group = NULL;
-			EC_POINT *cert_point = NULL;
-			char curve_name[80]; size_t curve_name_len = 0;
-			unsigned char pubkey[80]; size_t pubkey_len = 0;
+			char curve_name[80];
+			size_t curve_name_len = 0;
+			unsigned char pubkey[256];
+			size_t pubkey_len = 0;
 			int cert_nid = 0;
 			if (EVP_PKEY_get_group_name(o->key, curve_name, sizeof(curve_name), &curve_name_len) != 1 ||
 				(cert_nid = OBJ_txt2nid(curve_name)) == NID_undef ||
 				(cert_group = EC_GROUP_new_by_curve_name(cert_nid)) == NULL) {
-				fprintf(stderr, "Can not get EC_GROUP from EVP_PKEY");
-				EC_GROUP_free(ecgroup);
-				EC_POINT_free(ecpoint);
-				EC_GROUP_free(cert_group);
-				return -1;
+				debug_print(" [WARN %s ] Cannot get EC_GROUP from EVP_PKEY", o->id_str);
+				goto ec_out;
 			}
 			cert_point = EC_POINT_new(cert_group);
 			if (!cert_point ||
 				EVP_PKEY_get_octet_string_param(o->key, OSSL_PKEY_PARAM_PUB_KEY, pubkey, sizeof(pubkey), &pubkey_len) != 1 ||
 				EC_POINT_oct2point(cert_group, cert_point, pubkey, pubkey_len, NULL) != 1) {
-				fprintf(stderr, "Can not get EC_POINT from EVP_PKEY");
-				EC_GROUP_free(ecgroup);
-				EC_POINT_free(ecpoint);
-				EC_POINT_free(cert_point);
-				EC_GROUP_free(cert_group);
-				return -1;
+				debug_print(" [WARN %s ] Cannot get EC_POINT from EVP_PKEY", o->id_str);
+				goto ec_out;
 			}
 #endif
 			if (cert_nid != nid ||
@@ -539,12 +571,8 @@ int callback_public_keys(test_certs_t *objects,
 				debug_print(" [WARN %s ] Got different public"
 					"key then from the certificate",
 					o->id_str);
-				EC_GROUP_free(ecgroup);
-				EC_POINT_free(ecpoint);
-				return -1;
+				goto ec_out;
 			}
-			EC_GROUP_free(ecgroup);
-			EC_POINT_free(ecpoint);
 			o->verify_public = 1;
 		} else { /* store the public key for future use */
 			o->type = EVP_PKEY_EC;
@@ -555,44 +583,47 @@ int callback_public_keys(test_certs_t *objects,
 			EC_KEY_set_public_key(ec, ecpoint);
 			EC_KEY_set_group(ec, ecgroup);
 			EVP_PKEY_set1_EC_KEY(o->key, ec);
+			EC_KEY_free(ec);
 #else
 			ctx = EVP_PKEY_CTX_new_from_name(0, "EC", 0);
-			char curve_name[80]; size_t curve_name_len = 0;
-			unsigned char pubkey[80]; size_t pubkey_len = 0;
-			if (EVP_PKEY_get_group_name(o->key, curve_name, sizeof(curve_name), &curve_name_len)||
-				EVP_PKEY_get_octet_string_param(o->key, OSSL_PKEY_PARAM_PUB_KEY, pubkey, sizeof(pubkey), &pubkey_len) != 1) {
-				debug_print(" [WARN %s ] Can not get params from EVP_PKEY", o->id_str);
-				EC_GROUP_free(ecgroup);
-				EC_POINT_free(ecpoint);
-				return -1;
-			}
-			
+
+			const char *curve_name = OBJ_nid2sn(nid);
 			if (!(bld = OSSL_PARAM_BLD_new()) ||
-				OSSL_PARAM_BLD_push_utf8_string(bld, "group", curve_name, curve_name_len) != 1 ||
-				OSSL_PARAM_BLD_push_octet_string(bld, "pub", pubkey, pubkey_len) != 1 ||
+				OSSL_PARAM_BLD_push_utf8_string(bld, "group", curve_name, strlen(curve_name)) != 1 ||
+				OSSL_PARAM_BLD_push_octet_string(bld, "pub", pub, pub_len) != 1 ||
 				!(params = OSSL_PARAM_BLD_to_param(bld))) {
-				debug_print(" [WARN %s ] Can not set params from EVP_PKEY", o->id_str);
-				EC_GROUP_free(ecgroup);
-				EC_POINT_free(ecpoint);
-				OSSL_PARAM_BLD_free(bld);
-				EVP_PKEY_CTX_free(ctx);
-				return -1;
+				debug_print(" [WARN %s ] Cannot set params from EVP_PKEY", o->id_str);
+				goto ec_out;
 			}
-			OSSL_PARAM_BLD_free(bld);
 
 			if (ctx == NULL || params == NULL ||
 				EVP_PKEY_fromdata_init(ctx) != 1 ||
 				EVP_PKEY_fromdata(ctx, &o->key, EVP_PKEY_PUBLIC_KEY, params) != 1) {
-				debug_print(" [WARN %s ] Can not set params for EVP_PKEY", o->id_str);
-				EC_GROUP_free(ecgroup);
-				EC_POINT_free(ecpoint);
-				EVP_PKEY_CTX_free(ctx);
-				return -1;
+				debug_print(" [WARN %s ] Cannot set params for EVP_PKEY", o->id_str);
+				goto ec_out;
 			}
-			EVP_PKEY_CTX_free(ctx);
-			OSSL_PARAM_free(params);
 #endif
 		}
+
+		ec_error = 0;
+
+	ec_out:
+		ASN1_STRING_free(pub_asn1);
+		EC_GROUP_free(ecgroup);
+		EC_POINT_free(ecpoint);
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+		EVP_PKEY_CTX_free(ctx);
+		OSSL_PARAM_BLD_free(bld);
+		OSSL_PARAM_free(params);
+		EC_GROUP_free(cert_group);
+		EC_POINT_free(cert_point);
+#endif
+
+		if (ec_error) {
+			debug_print(" [WARN %s ] Failed to check EC public key", o->id_str);
+			return -1;
+		}
+
 	} else if (o->key_type == CKK_EC_EDWARDS
 		|| o->key_type == CKK_EC_MONTGOMERY) {
 		EVP_PKEY *key = NULL;
@@ -667,7 +698,7 @@ int callback_public_keys(test_certs_t *objects,
 		a = template[7].pValue;
 		os = d2i_ASN1_OCTET_STRING(NULL, &a, (long)template[7].ulValueLen);
 		if (!os) {
-			debug_print(" [WARN %s ] Can not decode EC_POINT", o->id_str);
+			debug_print(" [WARN %s ] Cannot decode EC_POINT", o->id_str);
 			return -1;
 		}
 		if (os->length != 32) {
@@ -689,7 +720,7 @@ int callback_public_keys(test_certs_t *objects,
 			/* TODO check EVP_PKEY type */
 
 			if (EVP_PKEY_get_raw_public_key(o->key, NULL, &publen) != 1) {
-				debug_print(" [WARN %s ] Can not get size of the key", o->id_str);
+				debug_print(" [WARN %s ] Cannot get size of the key", o->id_str);
 				ASN1_STRING_free(os);
 				return -1;
 			}
@@ -736,10 +767,16 @@ int callback_public_keys(test_certs_t *objects,
  * Store any secret keys
  */
 int callback_secret_keys(test_certs_t *objects,
-	CK_ATTRIBUTE template[], unsigned int template_size, CK_OBJECT_HANDLE object_handle)
+	CK_ATTRIBUTE template[], unsigned long template_size, CK_OBJECT_HANDLE object_handle)
 {
 	test_cert_t *o = NULL;
 
+	/* Ignore objects with empty ID and label that are left in SoftHSM after deriving key even after
+	 * destroying them */
+	if (template[13].pValue == NULL || template[1].pValue == NULL) {
+		return 0;
+	}
+
 	if ((o = add_object(objects, template[1], template[13])) == NULL)
 		return -1;
 
@@ -751,32 +788,36 @@ int callback_secret_keys(test_certs_t *objects,
 	o->private_handle = object_handle;
 	/* For verification/encryption, we use the same key */
 	o->public_handle = object_handle;
-	o->key_type = (template[0].ulValueLen != (CK_ULONG) -1)
+	o->key_type = (template[0].ulValueLen == sizeof(CK_KEY_TYPE))
 		? *((CK_KEY_TYPE *) template[0].pValue) : (CK_KEY_TYPE) -1;
-	o->sign = (template[3].ulValueLen != (CK_ULONG) -1)
+	o->sign = (template[3].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[3].pValue) : CK_FALSE;
-	o->sign = (template[4].ulValueLen != (CK_ULONG) -1)
+	o->verify = (template[4].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[4].pValue) : CK_FALSE;
-	o->encrypt = (template[5].ulValueLen != (CK_ULONG) -1)
+	o->encrypt = (template[5].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[5].pValue) : CK_FALSE;
-	o->decrypt = (template[6].ulValueLen != (CK_ULONG) -1)
+	o->decrypt = (template[6].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[6].pValue) : CK_FALSE;
-	o->derive_priv = (template[7].ulValueLen != (CK_ULONG) -1)
+	o->derive_priv = (template[7].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[7].pValue) : CK_FALSE;
-	o->wrap = (template[8].ulValueLen != (CK_ULONG) -1)
+	o->wrap = (template[8].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[8].pValue) : CK_FALSE;
-	o->unwrap = (template[9].ulValueLen != (CK_ULONG) -1)
+	o->unwrap = (template[9].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[9].pValue) : CK_FALSE;
-	o->extractable = (template[12].ulValueLen != (CK_ULONG) -1)
+	o->extractable = (template[12].ulValueLen == sizeof(CK_BBOOL))
 		? *((CK_BBOOL *) template[12].pValue) : CK_FALSE;
 
 	if (template[10].ulValueLen > 0) {
 		/* pass the pointer to our structure */
 		o->value = template[10].pValue;
 		template[10].pValue = NULL;
+		/* if there is CKA_VALUE_LEN it will be rewritten later */
+		o->bits = template[10].ulValueLen * 8;
 	}
 
-	o->bits = template[11].ulValueLen > 0 ? *((CK_ULONG *) template[11].pValue)*8 : 0;
+	if (template[11].pValue != NULL && template[11].ulValueLen > 0) {
+		o->bits = *((CK_ULONG *)template[11].pValue) * 8;
+	}
 
 	add_supported_mechs(o);
 
@@ -787,7 +828,7 @@ int callback_secret_keys(test_certs_t *objects,
 
 int search_objects(test_certs_t *objects, token_info_t *info,
 	CK_ATTRIBUTE filter[], CK_LONG filter_size, CK_ATTRIBUTE template[], CK_LONG template_size,
-	int (*callback)(test_certs_t *, CK_ATTRIBUTE[], unsigned int, CK_OBJECT_HANDLE))
+	int (*callback)(test_certs_t *, CK_ATTRIBUTE[], unsigned long, CK_OBJECT_HANDLE))
 {
 	CK_RV rv;
 	CK_FUNCTION_LIST_PTR fp = info->function_pointer;
@@ -847,7 +888,8 @@ int search_objects(test_certs_t *objects, token_info_t *info,
 			rv = fp->C_GetAttributeValue(info->session_handle, object_handles[i],
 				&(template[j]), 1);
 			if (rv == CKR_ATTRIBUTE_TYPE_INVALID ||
-			    rv == CKR_ATTRIBUTE_SENSITIVE) {
+			    rv == CKR_ATTRIBUTE_SENSITIVE ||
+			    rv == CKR_DEVICE_ERROR) {
 				continue;
 			} else if (rv != CKR_OK) {
 				fail_msg("C_GetAttributeValue: rv = 0x%.8lX\n", rv);
@@ -980,7 +1022,7 @@ void clean_all_objects(test_certs_t *objects) {
 	free(objects->data);
 }
 
-const char *get_mechanism_name(int mech_id)
+const char *get_mechanism_name(unsigned long mech_id)
 {
 	switch (mech_id) {
 		case CKM_RSA_PKCS:
@@ -995,6 +1037,14 @@ const char *get_mechanism_name(int mech_id)
 			return "SHA384_RSA_PKCS";
 		case CKM_SHA512_RSA_PKCS:
 			return "SHA512_RSA_PKCS";
+		case CKM_SHA3_224_RSA_PKCS:
+			return "SHA3_224_RSA_PKCS";
+		case CKM_SHA3_256_RSA_PKCS:
+			return "SHA3_256_RSA_PKCS";
+		case CKM_SHA3_384_RSA_PKCS:
+			return "SHA3_384_RSA_PKCS";
+		case CKM_SHA3_512_RSA_PKCS:
+			return "SHA3_512_RSA_PKCS";
 		case CKM_RSA_X_509:
 			return "RSA_X_509";
 		case CKM_ECDSA:
@@ -1009,6 +1059,14 @@ const char *get_mechanism_name(int mech_id)
 			return "ECDSA_SHA384";
 		case CKM_ECDSA_SHA512:
 			return "ECDSA_SHA512";
+		case CKM_ECDSA_SHA3_224:
+			return "ECDSA_SHA3_224";
+		case CKM_ECDSA_SHA3_256:
+			return "ECDSA_SHA3_256";
+		case CKM_ECDSA_SHA3_384:
+			return "ECDSA_SHA3_384";
+		case CKM_ECDSA_SHA3_512:
+			return "ECDSA_SHA3_512";
 		case CKM_EDDSA:
 			return "EDDSA";
 		case CKM_XEDDSA:
@@ -1041,18 +1099,36 @@ const char *get_mechanism_name(int mech_id)
 			return "SHA384_RSA_PKCS_PSS";
 		case CKM_SHA512_RSA_PKCS_PSS:
 			return "SHA512_RSA_PKCS_PSS";
+		case CKM_SHA3_224_RSA_PKCS_PSS:
+			return "SHA3_224_RSA_PKCS_PSS";
+		case CKM_SHA3_256_RSA_PKCS_PSS:
+			return "SHA3_256_RSA_PKCS_PSS";
+		case CKM_SHA3_384_RSA_PKCS_PSS:
+			return "SHA3_384_RSA_PKCS_PSS";
+		case CKM_SHA3_512_RSA_PKCS_PSS:
+			return "SHA3_512_RSA_PKCS_PSS";
 		case CKM_MD5_HMAC:
 			return "MD5_HMAC";
 		case CKM_SHA_1_HMAC:
 			return "SHA_1_HMAC";
+		case CKM_SHA_1_HMAC_GENERAL:
+			return "SHA_1_HMAC_GENERAL";
 		case CKM_SHA224_HMAC:
 			return "SHA224_HMAC";
+		case CKM_SHA224_HMAC_GENERAL:
+			return "SHA224_HMAC_GENERAL";
 		case CKM_SHA256_HMAC:
 			return "SHA256_HMAC";
+		case CKM_SHA256_HMAC_GENERAL:
+			return "SHA256_HMAC_GENERAL";
 		case CKM_SHA384_HMAC:
 			return "SHA384_HMAC";
+		case CKM_SHA384_HMAC_GENERAL:
+			return "SHA384_HMAC_GENERAL";
 		case CKM_SHA512_HMAC:
 			return "SHA512_HMAC";
+		case CKM_SHA512_HMAC_GENERAL:
+			return "SHA512_HMAC_GENERAL";
 		case CKM_RSA_PKCS_OAEP:
 			return "RSA_PKCS_OAEP";
 		case CKM_RIPEMD160:
@@ -1136,12 +1212,12 @@ const char *get_mechanism_name(int mech_id)
 		case CKM_AES_KEY_WRAP_PAD:
 			return "AES_KEY_WRAP_PAD";
 		default:
-			sprintf(name_buffer, "0x%.8X", mech_id);
+			sprintf(name_buffer, "0x%.8lX", mech_id);
 			return name_buffer;
 	}
 }
 
-const char *get_mgf_name(int mgf_id)
+const char *get_mgf_name(unsigned long mgf_id)
 {
 	switch (mgf_id) {
 		case CKG_MGF1_SHA1:
@@ -1154,13 +1230,41 @@ const char *get_mgf_name(int mgf_id)
 			return "MGF1_SHA384";
 		case CKG_MGF1_SHA512:
 			return "MGF1_SHA512";
+		case CKG_MGF1_SHA3_224:
+			return "MGF1_SHA3_224";
+		case CKG_MGF1_SHA3_256:
+			return "MGF1_SHA3_256";
+		case CKG_MGF1_SHA3_384:
+			return "MGF1_SHA3_384";
+		case CKG_MGF1_SHA3_512:
+			return "MGF1_SHA3_512";
 		default:
-			sprintf(name_buffer, "0x%.8X", mgf_id);
+			sprintf(name_buffer, "0x%.8lX", mgf_id);
 			return name_buffer;
 	}
 }
 
-const char *get_mechanism_flag_name(int mech_id)
+const char *
+get_key_type(test_cert_t * key)
+{
+	switch (key->key_type) {
+	case CKK_RSA:
+		return "RSA";
+	case CKK_EC:
+		return "EC";
+	case CKK_EC_EDWARDS:
+		return "EC_EDWARDS";
+	case CKK_EC_MONTGOMERY:
+		return "EC_MONTGOMERY";
+	case CKK_AES:
+		return "AES";
+	default:
+		sprintf(name_buffer, "0x%.8lX", key->key_type);
+		return name_buffer;
+	}
+}
+
+const char *get_mechanism_flag_name(unsigned long mech_id)
 {
 	switch (mech_id) {
 		case CKF_HW:
@@ -1212,18 +1316,48 @@ const char *get_mechanism_flag_name(int mech_id)
 		case CKF_EC_ECPARAMETERS:
 			return "CKF_EC_ECPARAMETERS";
 		default:
-			sprintf(flag_buffer, "0x%.8X", mech_id);
+			sprintf(flag_buffer, "0x%.8lX", mech_id);
 			return flag_buffer;
 	}
 }
 
+const char *
+get_mechanism_all_flag_name(unsigned long mech_id)
+{
+	CK_FLAGS j;
+	static char f_buffer[80];
+
+	f_buffer[0] = '\0';
+	for (j = 1; j <= CKF_EC_COMPRESS; j = j << 1)
+		/* append the name of the mechanism (only for known mechanisms) */
+		if ((mech_id & j) != 0 && strncmp("0x", get_mechanism_flag_name(j), 2)) {
+			snprintf(f_buffer + strlen(f_buffer),
+					sizeof(f_buffer) - strlen(f_buffer), "%s,", get_mechanism_flag_name(j));
+		}
+	/* remove comma at end of string */
+	if ((strlen(f_buffer) > 0) && f_buffer[strlen(f_buffer) - 1] == ',')
+		f_buffer[strlen(f_buffer) - 1] = '\0';
+	return f_buffer;
+}
+
 char *convert_byte_string(unsigned char *id, unsigned long length)
 {
 	unsigned int i;
-	char *data = malloc(3 * length * sizeof(char) + 1);
-	for (i = 0; i < length; i++)
-		sprintf(&data[i*3], "%02X:", id[i]);
-	data[length*3-1] = '\0';
+	char *data;
+	if (length == 0) {
+		return NULL;
+	}
+
+	data = malloc(3 * length * sizeof(char) + 1);
+	if (data == NULL) {
+		return NULL;
+	}
+
+	for (i = 0; i < length; i++) {
+		sprintf(&data[i * 3], "%02X:", id[i]);
+	}
+
+	data[length * 3 - 1] = '\0';
 	return data;
 }
 
@@ -1254,10 +1388,21 @@ void write_data_row(token_info_t *info, int cols, ...)
 
 int is_pss_mechanism(CK_MECHANISM_TYPE mech)
 {
-	return (mech == CKM_RSA_PKCS_PSS
-		|| mech == CKM_SHA1_RSA_PKCS_PSS
-		|| mech == CKM_SHA256_RSA_PKCS_PSS
-		|| mech == CKM_SHA384_RSA_PKCS_PSS
-		|| mech == CKM_SHA512_RSA_PKCS_PSS
-		|| mech == CKM_SHA224_RSA_PKCS_PSS);
+	return (mech == CKM_RSA_PKCS_PSS ||
+			mech == CKM_SHA1_RSA_PKCS_PSS ||
+			mech == CKM_SHA256_RSA_PKCS_PSS ||
+			mech == CKM_SHA384_RSA_PKCS_PSS ||
+			mech == CKM_SHA512_RSA_PKCS_PSS ||
+			mech == CKM_SHA224_RSA_PKCS_PSS ||
+			mech == CKM_SHA3_256_RSA_PKCS_PSS ||
+			mech == CKM_SHA3_384_RSA_PKCS_PSS ||
+			mech == CKM_SHA3_512_RSA_PKCS_PSS ||
+			mech == CKM_SHA3_224_RSA_PKCS_PSS);
+}
+
+CK_RV
+destroy_tmp_object(token_info_t *info, CK_OBJECT_HANDLE h)
+{
+	CK_FUNCTION_LIST_PTR fp = info->function_pointer;
+	return fp->C_DestroyObject(info->session_handle, h);
 }
